from numpy import *
from scipy.linalg import norm
import matplotlib as mpl
from matplotlib import pyplot
from os import system

def update_rcparams(fig_width_pt=700.0, fontsize=32, fontfamily='sans-serif'):
    inches_per_pt = 1.0/72.27               # Convert pt to inch
    golden_mean = 0.7#(sqrt(5)-1.0)/2.25         # Aesthetic ratio
    fig_width = fig_width_pt*inches_per_pt  # width in inches
    fig_height = fig_width*golden_mean      # height in inches
    fig_size =  [fig_width,fig_height]

    params = {'backend': 'ps',
              'font.family': fontfamily,    # can specify many more font and math font
              'font.serif':  'cm',          #properties, see ye ole info superhighway  
              'font.sans-serif' : 'arial',
              'font.size': fontsize,
              'axes.labelsize': fontsize,
              'axes.titlesize' : fontsize,
              'legend.fontsize': 24,
              'xtick.labelsize': fontsize,
              'ytick.labelsize': fontsize,
              'text.usetex': True,
              'figure.figsize': fig_size,
              'lines.linewidth' : 3,
              'lines.markersize'  : 20,
              'xtick.major.size':  10, 
              'xtick.minor.size' : 0,
              'ytick.major.size':  10, 
              'ytick.minor.size' : 0,
              # Makes latex all sans-serif, you have two options to do this
              # 'text.latex.preamble' : '\usepackage{sfmath}'}
              # or
              'mathtext.default' : 'regular'}
    return params
##
params = update_rcparams(fig_width_pt=850., fontfamily='sans-serif')
for key,entry in params.items():
    mpl.rcParams[key] = entry


errorp = zeros((1000-10,))
errorp2 = zeros((1000-10,))

for n in range(10, 1000):
    x = linspace(0,2*pi,n)
    h = 2*pi/(n-1)
    f = sin(x)
    fp = zeros( (x.shape[0]-2,) )
    fp2 = zeros( (x.shape[0]-4,) )
    for k in range(1,n-1):
       fp[k-1] = (1/h**2)*(f[k-1] - 2*f[k] + f[k+1])
    for k in range(2,n-2):   
       fp2[k-2] = (1/h**2)*((-1/12.)*f[k-2] + (4./3.)*f[k-1] + (-5./2.)*f[k] + (4./3.)*f[k+1] + (-1/12.)*f[k+2])

    fpexact = -sin(x)

    errorp[n-10] = norm(fpexact[1:-1] - fp)
    errorp2[n-10] = norm(fpexact[2:-2] - fp2)

n = arange(10,1000) 
pyplot.loglog(n,errorp)
pyplot.loglog(n,errorp2)
pyplot.loglog(n,1./n**2.0)
pyplot.loglog(n,1./n**4.0)
pyplot.legend(['FD Plot 1', 'FD Plot 2', 'Quadratic Ref', 'Quartic Ref'])

pyplot.grid(True, color='k', linestyle='-', linewidth=1)
pyplot.xlabel(r"$n$")
pyplot.ylabel("error norm")

pyplot.tight_layout(h_pad=0.05)
fname = 'FD_error.pdf'
# if using PNG, can set transparent=True in savefig
pyplot.savefig(fname, dpi=1000, format='pdf')
system('pdfcrop ' + fname + ' ' + fname) 
system('open -a preview FD_error.pdf')



