import numpy as np
from scipy import sparse
import time
from threading import Thread
import sys

def matvec(A,x,y,start,stop, myrank):
    '''
    Carry out the matrix-vector product y = A x 
    over the row-range [ start : stop] 
    This function will be called by multiple threads
    '''

    # Question 2: What are the dimensions of A, x, y here?
    # Question 3: Is the matrix A here the same as the A 
    #             matrix in the master thread portion?
    # Question 4: Are any race conditions possible here? 
    y[start:stop] = A*x
    
    # For printing inside of a threaded function, print to standard error
    sys.stderr.write("Thread " + str(myrank) + "\n")


##
# Master thread portion
##

# Construct Matrix 
n = 24000000
data = np.ones((3,n))
data[1,:] = -2.0
A = sparse.spdiags(data, [-1,0,1], n, n, format='csr')

# Question 1: Can you deduce how many cores your computer 
#           has by changing nt?

# nt sets the number of threads
nt = 4

# Data initialization
min_time = 10000
x = np.zeros((A.shape[0],))
y = np.zeros((A.shape[0],))

# This timing loop will do the threaded matrix-vector, and
# store the fastest time
for m in range(4):
    
    t_list = []
    for k in range(nt):
        start = int(k*(n/nt))       # compute thread k's starting row index (cast to int)
        stop = int((k+1)*(n/nt))    # compute thread k's stopping row index (cast to int)

        # Create (but DO NOT RUN) a threaded task equal to the function call
        # matvec(A[start:stop,:], x, y, start, stop, k)
        t_list.append(Thread(target=matvec, args=(A[start:stop,:], x, y, start, stop, k))) 

    # Fork and Join part, i.e., set each thread loose to run concurrently!
    start = time.perf_counter()
    for t in t_list:
        t.start()   
    for t in t_list:
        t.join()   
    end = time.perf_counter()
    min_time = min(end-start, min_time)

# Output result
print("\nThreaded mat-vec took %1.3e seconds with %d threads\n"%(min_time, nt))

