from numpy import *
from matplotlib import pyplot 
from poisson import poisson
from scipy.sparse.linalg import splu  # SPLU is a sparse matrix LU factorization
from scipy.sparse import eye as speye # speye generates a sparse identity matrix

##
# Problem Definition
#
# u_t = u_xx + u_yy,  on the unit box  
#
# u(t,x,y) = 0,       for (x,y) on the boundary
#
# u(t=0,x,y) = sin(pi x) * sin(pi y)
#
##

# Task Overview
# - Read through the code.  Make sure you understand it.  
# 
# - Finish the backward Euler function.  Then run the code, and 
#   make sure you understand the output.  Why is the solution 
#   decreasing in magnitude over time?
#
# - Implement forward Euler and RK2
#
# - Experiment with different ht values, observing when you violate the 
#   stability limit of forward Euler and RK2
#   --> You'll have to comment in the correct time stepping scheme down below
#       to switch between forward Euler, RK2, and backward Euler
#
# - If I told you that the  max eigenvalue of A was roughly 4.0/h^2, 
#   Could you pre-compute the largest allowed time-step size for forward Euler?
#


def euler_backward(A, y, ht):
    '''
    Carry out backward Euler's method for one time step
    '''
    I = speye(A.shape[0], format='csc')
    Euler_Mat = # Task fill in.  Note that I is an identity matrix 
                # of the same size as A

    Ainv = splu(Euler_Mat)  # Pay attention to the efficient sparse matrix LU used here
    return Ainv.solve(y)    # Pay attention to the interface for the sparse LU


def euler_forward(A, y, ht):
    '''
    Carry out forward Euler's method for one time step
    '''
    return y + # Task: Fill in with correct values! 


def RK2(A, y, ht):
    '''
    Carry out RK2 for one time step 
    '''

    k1 = # Task: Fill in with correct values!
    k2 = # Task: Fill in with correct values! 
    return y + # Task: Fill in with correct values! 


# Declare time domain 
# Tasks:  You will control and experiment with the size of ht, by increasing nt
t0 = 0.0
T = 1.0
nt = 101
ht = (T - t0)/float(nt-1)

# Declare spatial domain
n = 16
pts = linspace(0,1,n)
X,Y = meshgrid(pts, pts)
X = X.reshape(-1,)
Y = Y.reshape(-1,)
h = 1.0 / (n-1.0)

# Declare spatial discretization matrix
A = poisson((n,n), format='csc')
A = (1.0/h**2)*A

# Declare initial condition
#   This initial condition obeys an all 0 boundary condition. Moreover,
#   we have no forcing, meaning the boundary temperature is clamped to 0.
#
#   All this implies that the steady state solution (as time goes to infinity)
#   is what ???  
#   Task:  Tell me what the answer is here. 
y0 = sin(pi*X)*sin(pi*Y)

# Declare storage
y = zeros((nt,n*n))
y[0,:] = y0


# Run time-stepping
for i in range(1,nt):
    # Here is where you have to comment in / out the time-stepping scheme that you want

    # Task: you may have to adjust "nt" above so that ht is small 
    #       enough for forward Euler and RK2 to be stable.

    # Task: with a stable ht for forward Euler and RK2, you should 
    #       be able to get similar plots for all three ODE schemes

    # Option 1: Backward Euler
    y[i,:] = euler_backward(A, y[i-1,:], ht)
    
    # Option 2: Forward Euler
    #y[i,:] = euler_forward(A, y[i-1,:], ht)
    
    # Option 3: RK2
    #y[i,:] = RK2(A, y[i-1,:], ht)


##
# Plot three solution snapshots, at t_0, t_middle, and t_final
##
pyplot.figure(1)
pyplot.imshow(y[0,:].reshape(n,n), origin='lower', extent=(0, 1, 0, 1))
pyplot.colorbar()
pyplot.xlabel('X')
pyplot.ylabel('Y')
pyplot.title("Initial Condition")

pyplot.figure(2)
pyplot.imshow(y[int(nt/2),:].reshape(n,n))
pyplot.colorbar()
pyplot.xlabel('X')
pyplot.ylabel('Y')
pyplot.title("Solution at time T/2")

pyplot.figure(3)
pyplot.imshow(y[-1,:].reshape(n,n))
pyplot.colorbar()
pyplot.xlabel('X')
pyplot.ylabel('Y')
pyplot.title("Solution at final time")

pyplot.show()

