#!/bin/bash

########################################################################
# SBATCH arguments. Learn more by running "man sbatch"  
########################################################################

## This requests 1 node. 32 cores per node. Change this to 2, if you request 64 nodes.
#SBATCH --nodes=1

## This requests 4 tasks per node.
#SBATCH --ntasks-per-node 4   

## This is the max run-time allowed  hr:min:sec
#SBATCH --time=00:05:00 

## This is the filename for all printed output from your code
#SBATCH --output output_jacobi

## This is the filename for all printed error from your code
#SBATCH --error error_jacobi

## The debug partition will likely be faster, but if your code takes more than
## 8 processors, you need to switch to the general partition 
#SBATCH --partition debug

########################################################################
########################################################################


########################################################################
# Load MPI and your custom Conda environment
########################################################################
#    To create your Conda environment for this class, you can do...
#    $ module load miniconda<tab to complete version>  
#    $ conda create  –n mpi_numpy mpi mpi4py numpy 
#      < press y to accept installation of all new packages and the many dependencies > 

module load miniconda3
source activate mpi_numpy
########################################################################
########################################################################


########################################################################
# Now, change directory into the batch system's preferred scratch directory
# (don't worry about this now)
########################################################################
cd $SLURM_SUBMIT_DIR/
########################################################################
########################################################################


########################################################################
# Now, run your program.  Just change the below filename to run a 
# different program. 
########################################################################


srun --mpi=pmi2 python jacobi_oneD_MPI.py



